# Copyright (c) 2010-2011 Simplistix Ltd
# See license.txt for license details.

import os
import pkg_resources
import sys

from datetime import datetime
from zc.buildout import easy_install
from zc.buildout.easy_install import Installer
from zc.buildout.easy_install import logger
from zc.buildout.easy_install import IncompatibleVersionError
from zc.buildout.easy_install import default_versions
from zc.buildout.easy_install import is_distribute

required_by = {}
picked_versions = {}
links = {}  # eg:  {'jumbo-core': '~/.buildot/eggs/jumbo-codre...egg'}

# code to patch in
def _log_requirement(ws, req):
    for dist in sorted(ws):
        if req in dist.requires():
            req_ = str(req)
            if req_ not in required_by:
                required_by[req_] = set()
            required_by[req_].add(str(dist.as_requirement()))

original_get_dist = Installer._get_dist
def _get_dist(self, requirement, ws, always_unzip):
    global links
    dists = original_get_dist(self, requirement, ws, always_unzip)
        
    for dist in dists:
        links[dist.project_name] = dist.location
        if not (dist.precedence == pkg_resources.DEVELOP_DIST or \
                  (len(requirement.specs) == 1 and \
                       requirement.specs[0][0] == '==')):
            picked_versions[dist.project_name] = dist.version
    return dists

def _constrain(self, requirement):
    if is_distribute and requirement.key == 'setuptools':
        requirement = pkg_resources.Requirement.parse('distribute')
    version = self._versions.get(requirement.project_name.lower())
    if version:
        if version not in requirement:
            logger.error("The version, %s, is not consistent with the "
                         "requirement, %r.", version, str(requirement))
            raise IncompatibleVersionError("Bad version", version)

        requirement = pkg_resources.Requirement.parse(
            "%s[%s] ==%s" % (requirement.project_name,
                           ','.join(requirement.extras),
                           version))

    return requirement

file_name = None

def start(buildout):
    global file_name

    # normalise version case and make sure that this is stored in
    # buildout['versions']
    versions_section = buildout['buildout'].get('versions')
    current_versions = buildout.get(versions_section, {})
    for key,value in current_versions.items():
        lower_key = key.lower()
        if lower_key != key:
            del current_versions[key]  # optional, really
            current_versions[lower_key] = value

    python_version = current_versions.get('python')
    if python_version and python_version not in sys.version:
        raise IncompatibleVersionError(
            "python version specified not found in sys.version:",
            python_version
            )
        
    read_conf(buildout)

    # Apply the new versions in the Installer.  Whether this is needed
    # depends on the loading order of extensions.
    default_versions(current_versions)

    # patch methods
    easy_install._log_requirement = _log_requirement
    Installer._get_dist = _get_dist
    Installer._constrain = _constrain

def read_conf(buildout):
    global links_eggs, link_target

    links_eggs = buildout['buildout'].get('links-eggs', '').splitlines()
    if 'links-target' in buildout['buildout']:
        link_target = os.path.join(buildout['buildout']['directory'],
                                   buildout['buildout']['links-target'])
    else:
        link_target = os.path.join(buildout['buildout']['directory'], '.links')

    if not os.path.exists(link_target):
        os.mkdir(link_target)
    for link in os.listdir(link_target):
        os.unlink(os.path.join(link_target, link))
    

def make_link(buildout, pkg_name, egg_dir):

    if pkg_name in links_eggs:
        target = os.path.join(link_target, pkg_name)
        print "OK per ", pkg_name, egg_dir
        if os.path.islink(target):
            os.unlink(target)
        os.symlink(egg_dir, target)
    
    
    
def finish(buildout):
    # now check that buildout-versions is constrained!
    version_section = buildout['buildout'].get('versions')
    # if not (version_section and
    #         'buildout-versions' in buildout[version_section]):
    #     picked_versions['buildout-versions']=pkg_resources.require(
    #         'buildout-versions'
    #         )[0].version

    if links:
        for pkg_name, egg_dir in links.iteritems():
            make_link(buildout, pkg_name, egg_dir)
    return
    # if picked_versions:
    #     output = ['[versions]']
    #     required_output = []
    #     for dist_, version in sorted(picked_versions.items()):
    #         if dist_ in required_by:
    #             required_output.append('')
    #             required_output.append('# Required by:')
    #             for req_ in sorted(required_by[dist_]):
    #                 required_output.append('# '+req_)
    #             target = required_output
    #         else:
    #             target = output
    #         target.append("%s = %s" % (dist_, version))

    #     output.extend(required_output)
        
    #     print "Versions had to be automatically picked."
    #     print "The following part definition lists the versions picked:"
    #     print '\n'.join(output)
    #     if file_name:
    #         if os.path.exists(file_name):
    #             output[:1] = [
    #                 '',
    #                 '# Added by Buildout Versions at %s' % datetime.now(),
    #                 ]
    #         output.append('')
    #         f = open(file_name,'a')
    #         f.write('\n'.join(output))
    #         f.close()
    #         print "This information has been written to %r" % file_name
